package de.tu_darmstadt.tk.SmartHomeNetworkSim.view;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.util.Collection;
import java.util.Iterator;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JPanel;

import de.tu_darmstadt.tk.SmartHomeNetworkSim.control.Controller;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Connection;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Model;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Port;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.SmartDevice;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.protocols.MQTT_protocol;

@SuppressWarnings("serial")
public class VisualisationPanel extends JPanel implements Observer {

	/**
	 * Smart Home model which is visualized
	 */
	private Model model;

	/**
	 * Controller to notify when the model changes
	 */
	private Controller control;

	/**
	 * Listener which processes the GUI Interactions
	 */
	private VisualisationInteractor interactor;

	/**
	 * Initializes the Visualization Panel
	 * 
	 * @param model
	 *            Model to visualize
	 * @param control
	 *            Control, which changes the model
	 */
	public VisualisationPanel(Model model, Controller control) {
		super();

		this.model = model;
		this.control = control;
		
		this.interactor = new VisualisationInteractor(model, control, this);

		this.addMouseMotionListener(interactor);
		this.addMouseListener(interactor);
	}

	/**
	 * Performs further initializations, which have to be performed, after the
	 * model was made visible
	 */
	public void delayedInit() {
		this.addComponentListener(new ComponentListener() {

			@Override
			public void componentShown(ComponentEvent e) {
			}

			@Override
			public void componentResized(ComponentEvent e) {
				control.setDimension(getWidth(), getHeight(), model.getDepth(),
						true);
				repaint();
			}

			@Override
			public void componentMoved(ComponentEvent e) {
			}

			@Override
			public void componentHidden(ComponentEvent e) {
			}
		});
		control.setDimension(getWidth(), getHeight(), model.getDepth(),
				true);
		repaint();
	}

	@Override
	public void paint(Graphics g) {
		// paint white background
		g.setColor(Color.white);
		g.fillRect(0, 0, this.getWidth(), this.getHeight());

		paintConnections(g);

		paintDevices(g);
	}

	/**
	 * Paints the smart devices of the Model
	 * 
	 * @param g
	 *            the Graphics object to visualize on
	 */
	public void paintDevices(Graphics g) {

		for (Iterator<SmartDevice> it = model.getDevices().iterator();it.hasNext();) {
			SmartDevice s = it.next();
			int x = s.getX();
			int y = s.getY();
			if (s == interactor.dragged) {
				// Update visualization of dragged object
				x = interactor.dragged_x;
				y = interactor.dragged_y;
			}
			g.setColor(Color.WHITE);
			g.fillOval(x - control.getDevice_visualization_radius(), y - control.getDevice_visualization_radius(),
					2 * control.getDevice_visualization_radius() - 1, 2 * control.getDevice_visualization_radius() - 1);
			g.setColor(Color.BLACK);
			g.drawOval(x - control.getDevice_visualization_radius(), y - control.getDevice_visualization_radius(),
					2 * control.getDevice_visualization_radius() - 1, 2 * control.getDevice_visualization_radius() - 1);
			g.setColor(Color.BLUE);
			g.drawOval(x - control.getDevice_visualization_radius() + 2,
					y - control.getDevice_visualization_radius() + 2, 2 * control.getDevice_visualization_radius() - 5,
					2 * control.getDevice_visualization_radius() - 5);
			g.setColor(Color.BLACK);
			g.drawString(s.getName(),
					x - g.getFontMetrics().stringWidth(s.getName()) / 2, y
							+ control.getDevice_visualization_radius() + 11);
		}
	}

	/**
	 * Paints the Connections
	 * 
	 * @param g
	 *            the Graphics object to visualize on
	 */
	private void paintConnections(Graphics g) {
		// For all Connections
		for (Connection c : model.getConnections()) {
			Color connectionState;
			switch (c.getStatus()) {
			case Connection.ACTIVE:
				connectionState = Color.GREEN;
				break;
			case Connection.HALTED:
				connectionState = Color.ORANGE;
				break;
			default:
				connectionState = Color.GREEN;
				break;
			}
			/**
			 * All Devices that are part of the connection
			 */
			Collection<Port> d = c.getParticipants();
			if(d.size()==0){
				System.out.println("WARNING: Empty, undeleted Connection:"+c.toString());
				continue;
			}
			if(c.getProtocol() instanceof MQTT_protocol){
				Port broker = c.getProtocol().getDevicesWithRole(0).iterator().next();
				int broker_x = 0, broker_y = 0;
				if(broker==null){
					/**
					 * if broker was removed, paint broker at the average position
					 */
					for(Port p: c.getParticipants()){
						if(p.getOwner()==interactor.dragged){
							broker_x+=interactor.dragged_x;
							broker_y+=interactor.dragged_y;
						}else{

							broker_x+=p.getOwner().getX();
							broker_y+=p.getOwner().getY();
						}
					}
					broker_x/=c.getParticipants().size();
					broker_y/=c.getParticipants().size();
					g.fillOval(broker_x-control.getDevice_visualization_radius()/4, broker_y-control.getDevice_visualization_radius()/4, control.getDevice_visualization_radius()/2, control.getDevice_visualization_radius()/2);
				}else{
					broker_x=broker.getOwner().getX();
					broker_y=broker.getOwner().getY();
				}
				for(Port sd:d){
					if(!model.getDevices().contains(sd.getOwner())){
						SmartDevice removed = sd.getOwner();
						g.setColor(Color.RED);
						g.fillOval(removed.getX()-control.getDevice_visualization_radius()/4, removed.getY()-control.getDevice_visualization_radius()/4, control.getDevice_visualization_radius()/2, control.getDevice_visualization_radius()/2);
					}else{
						g.setColor(connectionState);
					}
					
					if (broker!= sd) {
						// Check if dragged object
						int s_x, s_y, sd_x, sd_y;
						if (broker != null && broker.getOwner() == interactor.dragged) {
							s_x = interactor.dragged_x;
							s_y = interactor.dragged_y;
						} else if(broker==null){
							/**
							 * if broker was removed, paint edges to the previously calculated average position
							 */
							s_x=broker_x;
							s_y=broker_y;
						}else{
							s_x = broker_x;
							s_y = broker_y;
						}
						if (sd.getOwner() == interactor.dragged) {
							sd_x = interactor.dragged_x;
							sd_y = interactor.dragged_y;
						} else {
							sd_x = sd.getOwner().getX();
							sd_y = sd.getOwner().getY();
						}
						g.drawLine(s_x, s_y, sd_x, sd_y);
					}
				}
			}
			else if (d.size() == 2) {
				Port s = c.getSource();
				for (Port sd : d) {
					if(!model.getDevices().contains(sd.getOwner())){
						SmartDevice removed = sd.getOwner();
						g.fillOval(removed.getX()-control.getDevice_visualization_radius()/4, removed.getY()-control.getDevice_visualization_radius()/4, control.getDevice_visualization_radius()/2, control.getDevice_visualization_radius()/2);
					}
					if (s != sd) {
						// Check if dragged object
						int s_x, s_y, sd_x, sd_y;
						if (s.getOwner() == interactor.dragged) {
							s_x = interactor.dragged_x;
							s_y = interactor.dragged_y;
						} else {
							s_x = s.getOwner().getX();
							s_y = s.getOwner().getY();
						}
						if (sd.getOwner() == interactor.dragged) {
							sd_x = interactor.dragged_x;
							sd_y = interactor.dragged_y;
						} else {
							sd_x = sd.getOwner().getX();
							sd_y = sd.getOwner().getY();
						}
						g.drawLine(s_x, s_y, sd_x, sd_y);
					}
				}
			} else if (d.size() == 1) {

			} else if (d.size() <= 0) {
				// Invalid Connection
			} else {
				// Draw MultiConnection
			}

		}
		// paint new in progress connection, if a connection or link is in
		// creation
		g.setColor(Color.ORANGE);
		if (interactor.connectionFrom != null)
			g.drawLine(interactor.connectionFrom.getX(),
					interactor.connectionFrom.getY(), interactor.dragged_x,
					interactor.dragged_y);
	}

	@Override
	public void update(Observable o, Object arg) {
		repaint();
	}

}
